@@ -14,7 +14,7 @@ structure(c("Negative Control", "Dextrin", "D-Maltose", "D-Trehalose",
 "L-Aspartic Acid", "L-Glutamic Acid", "L-Histidine", "L-Pyroglutamic Acid", 
 "L-Serine", "Lincomycin", "Guanidine Hydrochloride", "Niaproof 4", 
 "Pectin", "D-Galacturonic Acid", "L-Galactonic Acid-g-Lactone", 
-"D-Gluconic Acid", "D-Glucuronic", "Glucuronamide", "Mucic Acid", 
+"D-Gluconic Acid", "D-Glucuronic Acid", "Glucuronamide", "Mucic Acid", 
 "Quinic Acid", "D-Saccharic Acid", "Vancomycin", "Tetrazolium Violet", 
 "Tetrazolium Blue", "p-Hydroxy-Phenylacetic Acid", "Methyl Pyruvate", 
 "D-Lactic Acid Methyl Ester", "L-Lactic Acid", "Citric Acid", 
@@ -268,7 +268,7 @@ structure(c("Negative Control", "Dextrin", "D-Maltose", "D-Trehalose",
 "Bleomycin", "Bleomycin", "Bleomycin", "Colistin", "Colistin", 
 "Colistin", "Colistin", "Minocycline", "Minocycline", "Minocycline", 
 "Minocycline", "Capreomycin", "Capreomycin", "Capreomycin", "Capreomycin", 
-"Demeclocyline", "Demeclocyline", "Demeclocyline", "Demeclocyline", 
+"Demeclocycline", "Demeclocycline", "Demeclocycline", "Demeclocycline", 
 "Nafcillin", "Nafcillin", "Nafcillin", "Nafcillin", "Cefazolin", 
 "Cefazolin", "Cefazolin", "Cefazolin", "Enoxacin", "Enoxacin", 
 "Enoxacin", "Enoxacin", "Nalidixic Acid", "Nalidixic Acid", "Nalidixic Acid", 
